//
//  CWMapViewController.m
//  ResaISkane
//
//  Created by Fredrik Olsson on 2010-02-05.
//  Copyright 2010 Fredrik Olsson. All rights reserved.
//

#import "CWMapViewController.h"
#import "NSDate+CWExtentions.h"
#import "CSRouteAnnotation.h"
#import "CSRouteView.h"

@interface CWJourneyAnnotation : NSObject <MKAnnotation> {
@private
  CLLocationCoordinate2D coordinate;
  CWPointType pointType;
  MKPinAnnotationColor pinColor;
  NSString* title;
  NSString* subtitle;
}

@property(nonatomic, assign) CLLocationCoordinate2D coordinate;
@property(nonatomic, assign) CWPointType pointType;
@property(nonatomic, assign) MKPinAnnotationColor pinColor;
@property(nonatomic, copy) NSString* title;
@property(nonatomic, copy) NSString* subtitle;

+(CWJourneyAnnotation*)journeyAnnotationWithStartRouteLink:(CWRouteLink*)routeLink;
+(CWJourneyAnnotation*)journeyAnnotationWithRouteLink:(CWRouteLink*)routeLink previousRouteLink:(CWRouteLink*)prevRouteLink;
+(CWJourneyAnnotation*)journeyAnnotationWithEndRouteLink:(CWRouteLink*)routeLink;

@end


@implementation CWMapViewController

-(id)initWithPoint:(CWPoint*)aPoint;
{
  self = [self init];
  if (self) {
    point = [aPoint retain];
    annotations = [[NSMutableArray alloc] init];
    self.title = point.title;
  }
  return self;
}

-(id)initWithJourney:(CWJourney*)aJourney;
{
  self = [self init];
  if (self) {
    annotations = [[NSMutableArray alloc] init];
    journey = [aJourney retain];
  }
  return self;
}

-(void)loadView;
{
	if (point != nil) {
		mapView = [[MKMapView alloc] initWithFrame:CGRectMake(0, 0, 320, 400)];
    self.view = mapView;
    [self performSelectorInBackground:@selector(primePointAnnotation) withObject:nil];
  } else if (journey != nil) {
		[[NSBundle mainBundle] loadNibNamed:@"CWMapViewController" owner:self options:nil];
    fromTitleLabel.text = NSLocalizedString(@"FromTitle", nil);
    toTitleLabel.text = NSLocalizedString(@"ToTitle", nil);
    [fromTitleLabel sizeToFit];
    [toTitleLabel sizeToFit];
    CGFloat maxWidth = MAX(fromTitleLabel.bounds.size.width, toTitleLabel.bounds.size.width);
    CGRect frame = fromLabel.frame;
    CGFloat oldX = frame.origin.x;
    frame.origin.x = maxWidth + 21;
    frame.size.width -= (frame.origin.x - oldX);
    fromLabel.frame = frame;
    frame.origin.y += 23;
    toLabel.frame = frame;
  	[self performSelectorInBackground:@selector(primeJourneyAnnotations) withObject:nil];
  }
}

-(void)viewDidLoad;
{
  CWPoint* from = journey.from;
  CWPoint* to = journey.to;
  fromLabel.text = from.title;
  if (from.ID <= CWPointCurrentLocationID) {
  	fromLabel.textColor = [UIColor blueColor];
  }
	toLabel.text = to.title;
  if (to.ID <= CWPointCurrentLocationID) {
  	toLabel.textColor = [UIColor blueColor];
  }
}

-(IBAction)targetSwap:(UIButton*)sender;
{
  sender.selected = !sender.selected;
  mapView.showsUserLocation = sender.selected;
}

-(void)primePointAnnotation;
{
  NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
  point.coordinate;
  hasGoogleMapsCoordinate = (point.coordinateSource == CWPointCoordinateSourceGoogleMaps);
  [annotations addObject:point];
  [self performSelectorOnMainThread:@selector(addAnnotations) withObject:nil waitUntilDone:NO];
  [pool release];
}

static BOOL tempHasGoogleMapsCoordinate;

-(void)primeJourneyAnnotations;
{
  NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
	tempHasGoogleMapsCoordinate = NO;
  points = [[NSMutableArray alloc] init];
	CWJourneyAnnotation* annotation = [CWJourneyAnnotation journeyAnnotationWithStartRouteLink:[journey.routeLinks objectAtIndex:0]];
	[annotations addObject:annotation];
  [points addObject:[[[CLLocation alloc] initWithLatitude:annotation.coordinate.latitude longitude:annotation.coordinate.longitude] autorelease]];
  for (int i = 0; i < [journey.routeLinks count] - 1; i++) {
  	annotation = [CWJourneyAnnotation journeyAnnotationWithRouteLink:[journey.routeLinks objectAtIndex:i + 1] previousRouteLink:[journey.routeLinks objectAtIndex:i]];
		[annotations addObject:annotation];
    [points addObject:[[[CLLocation alloc] initWithLatitude:annotation.coordinate.latitude longitude:annotation.coordinate.longitude] autorelease] ];
  }
  annotation = [CWJourneyAnnotation journeyAnnotationWithEndRouteLink:[journey.routeLinks lastObject]];
	[annotations addObject:annotation];
  [points addObject:[[[CLLocation alloc] initWithLatitude:annotation.coordinate.latitude longitude:annotation.coordinate.longitude] autorelease]];

  hasGoogleMapsCoordinate = tempHasGoogleMapsCoordinate;

  [self performSelectorOnMainThread:@selector(addAnnotations) withObject:nil waitUntilDone:NO];

  [pool release];
}

-(void)addGoogleMapsDesclaimer;
{
	NSLog(@"Satan i gatan!");
  NSString* disclaimer = NSLocalizedString(@"CoordinateDisclaimer", nil);
  UILabel* label = [[UILabel alloc] initWithFrame:CGRectZero];
  label.alpha = 0;
  label.numberOfLines = 0;
  label.font = [UIFont systemFontOfSize:14.0f];
  label.textAlignment = UITextAlignmentCenter;
  label.textColor = [UIColor whiteColor];
  label.backgroundColor = [UIColor colorWithRed:0.6f green:0.65f blue:0.7f alpha:0.9f];
  label.shadowColor = [UIColor grayColor];
  label.shadowOffset = CGSizeMake(0, -1);
  label.text = disclaimer;
  CGRect frame = mapView.bounds;
  CGSize size = [disclaimer sizeWithFont:label.font constrainedToSize:frame.size];
  frame.origin.y = frame.size.height - size.height;
  frame.size.height = size.height;
  label.frame = frame;
  [mapView addSubview:label];
  [UIView beginAnimations:@"dadeInDisclaimer" context:NULL];
	label.alpha = 1;
  [UIView commitAnimations];
  [label release];
}

-(void)addAnnotations;
{
  if (points != nil) {
    CSRouteAnnotation* routeAnnotation = [[[CSRouteAnnotation alloc] initWithPoints:points] autorelease];
    [mapView addAnnotation:routeAnnotation];
  }
  MKCoordinateRegion region;
	CLLocationCoordinate2D max = (CLLocationCoordinate2D) {-1000, -1000};
	CLLocationCoordinate2D min = (CLLocationCoordinate2D) {1000, 1000};
  for (id<MKAnnotation> annotation in annotations) {
		CLLocationCoordinate2D crd = annotation.coordinate;
    max = (CLLocationCoordinate2D){MAX(max.latitude, crd.latitude), MAX(max.longitude, crd.longitude)};
    min = (CLLocationCoordinate2D){MIN(min.latitude, crd.latitude), MIN(min.longitude, crd.longitude)};
    [mapView addAnnotation:annotation];
  }
  if ([annotations count] == 1) {
    region = MKCoordinateRegionMake(max, MKCoordinateSpanMake(0.01, 0.01));
  } else {
    CLLocationCoordinate2D center = (CLLocationCoordinate2D) {(min.latitude + max.latitude) / 2, (min.longitude + max.longitude) / 2};
		MKCoordinateSpan span = MKCoordinateSpanMake((max.latitude - min.latitude), (max.longitude - min.longitude));
    region = MKCoordinateRegionMake(center, span);
  }
  [mapView setRegion:region];
  if (hasGoogleMapsCoordinate) {
  	[self addGoogleMapsDesclaimer];
  }
}

- (void)mapView:(MKMapView *)mapView regionWillChangeAnimated:(BOOL)animated
{
  routeView.hidden = YES;
}

- (void)mapView:(MKMapView *)mapView regionDidChangeAnimated:(BOOL)animated
{
  routeView.hidden = NO;
  [routeView regionChanged];
}

- (MKAnnotationView *)mapView:(MKMapView *)aMapView viewForAnnotation:(id <MKAnnotation>)annotation;
{
  if (annotation == aMapView.userLocation) {
    return nil;
  } else if ([annotation isKindOfClass:[CSRouteAnnotation class]]) {
		CSRouteAnnotation* routeAnnotation = (CSRouteAnnotation*) annotation;

    routeAnnotation.lineColor = [UIColor colorWithRed:0 green:0 blue:1 alpha:.5f];

    routeView = [[[CSRouteView alloc] initWithFrame:CGRectMake(0, 0, mapView.frame.size.width, mapView.frame.size.height)] autorelease];

    routeView.annotation = routeAnnotation;
    routeView.mapView = mapView;

    return routeView;
	} else {
    static NSString* ReuseId = @"ReuseId";
    MKPinAnnotationView* view = (MKPinAnnotationView*)[aMapView dequeueReusableAnnotationViewWithIdentifier:ReuseId];
    CWPointType pointType;
    MKPinAnnotationColor pinColor = MKPinAnnotationColorPurple;

    if ([annotation isKindOfClass:[CWPoint class]]) {
      pointType = point.type;
    } else if ([annotation isKindOfClass:[CWJourneyAnnotation class]]) {
      CWJourneyAnnotation* journeyAnnotation = (CWJourneyAnnotation*)annotation;
      pointType = journeyAnnotation.pointType;
      pinColor = journeyAnnotation.pinColor;
    }

    if (view == nil) {
      view = [[[MKPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:ReuseId] autorelease];
      view.animatesDrop = YES;
      view.canShowCallout = YES;
    }

    UIImageView* iconView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:[NSString stringWithFormat:@"call_point-%d.png", pointType]]];
    view.leftCalloutAccessoryView = iconView;
    [iconView release];
    view.pinColor = pinColor;
    view.annotation = annotation;

    return view;
  }
}

- (void)dealloc;
{
  [point release];
  [journey release];
  [annotations release];
  [points release];
  [super dealloc];
}


@end


@implementation CWJourneyAnnotation

@synthesize coordinate, pointType, pinColor, title, subtitle;

+(void)setupAnnotation:(CWJourneyAnnotation*)annotation withPoint:(CWPoint*)point;
{
  annotation.coordinate = point.coordinate;
	tempHasGoogleMapsCoordinate |= (point.coordinateSource == CWPointCoordinateSourceGoogleMaps);
  annotation.pointType = point.type;
	annotation.title = point.title;
}

+(CWJourneyAnnotation*)journeyAnnotationWithStartRouteLink:(CWRouteLink*)routeLink;
{
	CWJourneyAnnotation* annotation = [[[CWJourneyAnnotation alloc] init] autorelease];
  [self setupAnnotation:annotation withPoint:routeLink.from];
  annotation.pinColor = MKPinAnnotationColorGreen;
  annotation.subtitle = [NSString stringWithFormat:NSLocalizedString(@"SubtitleStart", nil), [routeLink.actualDeparture localizedShortTimeString], routeLink.line.fullName];
  return annotation;
}

+(CWJourneyAnnotation*)journeyAnnotationWithRouteLink:(CWRouteLink*)routeLink previousRouteLink:(CWRouteLink*)prevRouteLink;
{
	CWJourneyAnnotation* annotation = [[[CWJourneyAnnotation alloc] init] autorelease];
  [self setupAnnotation:annotation withPoint:routeLink.from];
  annotation.pinColor = MKPinAnnotationColorRed;
  annotation.subtitle = [NSString stringWithFormat:NSLocalizedString(@"SubtitleStart", nil), [routeLink.actualDeparture localizedShortTimeString], routeLink.line.fullName];
//  annotation.subtitle = [NSString stringWithFormat:NSLocalizedString(@"SubtitleMiddle", nil), [prevRouteLink.actualArrival localizedShortTimeString], prevRouteLink.line.fullName, [routeLink.actualDeparture localizedShortTimeString], routeLink.line.fullName];
  return annotation;
}

+(CWJourneyAnnotation*)journeyAnnotationWithEndRouteLink:(CWRouteLink*)routeLink;
{
	CWJourneyAnnotation* annotation = [[[CWJourneyAnnotation alloc] init] autorelease];
  [self setupAnnotation:annotation withPoint:routeLink.to];
  annotation.pinColor = MKPinAnnotationColorRed;
  annotation.subtitle = [NSString stringWithFormat:NSLocalizedString(@"SubtitleEnd", nil), [routeLink.actualArrival localizedShortTimeString], routeLink.line.fullName];
  return annotation;
}


@end


